@@ -42,6 +42,7 @@ struct call {
 	char *aluri;              /**< Alert-Info URI                       */
 	char *local_uri;          /**< Local SIP uri                        */
 	char *local_name;         /**< Local display name                   */
+	char *contact_uri;        /**< Peer Contact SIP Address             */
 	char *peer_uri;           /**< Peer SIP Address                     */
 	char *peer_name;          /**< Peer display name                    */
 	struct sa msg_src;        /**< Peer message source address          */
@@ -346,6 +347,7 @@ static void call_destructor(void *arg)
 	mem_deref(call->id);
 	mem_deref(call->local_uri);
 	mem_deref(call->local_name);
+	mem_deref(call->contact_uri);
 	mem_deref(call->peer_uri);
 	mem_deref(call->peer_name);
 	mem_deref(call->replaces);
@@ -725,6 +727,9 @@ static void call_decode_sip_autoanswer(struct call *call,
 
 int call_streams_alloc(struct call *call)
 {
+	if (!call)
+		return EINVAL;
+
 	struct account *acc = call->acc;
 	struct stream_param strm_prm;
 	struct le *le;
@@ -792,6 +797,12 @@ int call_streams_alloc(struct call *call)
 			return err;
 	}
 
+	if (call->cfg->audio.level && !call->got_offer) {
+		err = audio_enable_level(call->audio);
+		if (err)
+			return err;
+	}
+
 	return 0;
 }
 
@@ -862,8 +873,14 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	if (local_name)
 		err |= str_dup(&call->local_name, local_name);
 
-	if (msg)
+	if (msg) {
+		struct sip_addr addr;
+		const struct sip_hdr *hdr = sip_msg_hdr(msg, SIP_HDR_CONTACT);
+		if (hdr && 0 == sip_addr_decode(&addr, &hdr->val))
+			err |= pl_strdup(&call->contact_uri, &addr.auri);
+
 		err |= pl_strdup(&call->peer_uri, &msg->from.auri);
+	}
 
 	if (err)
 		goto out;
@@ -1093,15 +1110,17 @@ int call_modify(struct call *call)
 	debug("call: modify\n");
 
 	if (call_refresh_allowed(call)) {
-		err = call_sdp_get(call, &desc, true);
-		if (!err) {
-			bevent_call_emit(BEVENT_CALL_LOCAL_SDP, call,
-					 "offer");
+		err = bevent_call_emit(BEVENT_CALL_LOCAL_SDP, call, "offer");
+		if (err)
+			return err;
 
-			err = sipsess_modify(call->sess, desc);
-			if (err)
-				goto out;
-		}
+		err = call_sdp_get(call, &desc, true);
+		if (err)
+			return err;
+
+		err = sipsess_modify(call->sess, desc);
+		if (err)
+			goto out;
 	}
 
 	err = call_update_media(call);
@@ -1521,6 +1540,8 @@ const char *call_id(const struct call *call)
 /**
  * Get the URI of the peer
  *
+ * For incoming calls, this is the From header URI of the incoming INVITE.
+ *
  * @param call  Call object
  *
  * @return Peer URI
@@ -1531,6 +1552,25 @@ const char *call_peeruri(const struct call *call)
 }
 
 
+/**
+ * Get the Contact URI of the peer
+ *
+ * For outgoing calls, this is the same as call_peeruri.
+ * For incoming calls, this is the Contact header URI of the incoming INVITE.
+ *
+ * @param call  Call object
+ *
+ * @return Peer Contact URI
+ */
+const char *call_contacturi(const struct call *call)
+{
+	if (!call)
+		return NULL;
+
+	return call->outgoing ? call->peer_uri : call->contact_uri;
+}
+
+
 /**
  * Get the local URI of the call
  *
@@ -2511,6 +2551,9 @@ static int sipsess_desc_handler(struct mbuf **descp, const struct sa *src,
 			return err;
 
 		call_set_mdir(call, call->estadir, call->estvdir);
+		err = bevent_call_emit(BEVENT_CALL_LOCAL_SDP, call, "offer");
+		if (err)
+			return err;
 	}
 
 	err = call_sdp_get(call, descp, true);
@@ -2523,7 +2566,7 @@ static int sipsess_desc_handler(struct mbuf **descp, const struct sa *src,
 	     (*descp)->buf, (*descp)->end);
 #endif
 
-	return err;
+	return 0;
 }
 
 
@@ -2546,6 +2589,12 @@ static int send_invite(struct call *call)
 
 	routev[0] = account_outbound(call->acc, 0);
 
+	if (!list_isempty(&call->streaml)) {
+		err = bevent_call_emit(BEVENT_CALL_LOCAL_SDP, call, "offer");
+		if (err)
+			return err;
+	}
+
 	err = sipsess_connect(&call->sess, uag_sipsess_sock(),
 			      call->peer_uri,
 			      call->local_name,
@@ -2584,8 +2633,6 @@ static int send_invite(struct call *call)
 	/* save call setup timer */
 	call->time_conn = time(NULL);
 
-	bevent_call_emit(BEVENT_CALL_LOCAL_SDP, call, "offer");
-
 	return 0;
 }
 
@@ -2632,6 +2679,9 @@ bool call_early_video_available(const struct call *call)
 	struct le *le;
 	struct sdp_media *v;
 
+	if (!call)
+		return false;
+
 	LIST_FOREACH(sdp_session_medial(call->sdp, false), le) {
 		v = le->data;
 		if (0 == str_cmp(sdp_media_name(v), "video") &&
@@ -2727,6 +2777,9 @@ int call_reset_transp(struct call *call, const struct sa *laddr)
 
 const struct sa *call_laddr(const struct call *call)
 {
+	if (!call)
+		return NULL;
+
 	return sdp_session_laddr(call->sdp);
 }
 
